@@ -232,9 +232,18 @@ class CascadeSummary(CascadeBaseState):
         if self.context.config_dump_text_blocks:
             super().on_exit()
 
-        # parsed object
-        self.context.cascade_summary = cascade_summary.parse(self.textbuf.getvalue(),
-                self.context.config_debug_parser)
+        # parse block-separated text
+        try:
+            self.context.cascade_summary = cascade_summary.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_summary = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 
 class CascadeDetailInitialDefect(BaseState):
@@ -320,9 +329,18 @@ class CascadeDetailProjectile(CascadeBaseState):
         if self.context.config_dump_text_blocks:
             super().on_exit()
 
-        # parsed object
-        self.context.cascade_detail_projectile = cascade_detail_projectile.parse(self.textbuf.getvalue(),
-                self.context.config_debug_parser)
+        # parse block-separated text
+        try:
+            self.context.cascade_detail_projectile = cascade_detail_projectile.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_detail_projectile = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 
 class CascadeReportSummary(CascadeBaseState):
@@ -336,9 +354,18 @@ class CascadeReportSummary(CascadeBaseState):
         if self.context.config_dump_text_blocks:
             super().on_exit()
 
-        # parsed object
-        self.context.cascade_report_summary = cascade_report_summary.parse(self.textbuf.getvalue(),
-                self.context.config_debug_parser)
+        # parse block-separated text
+        try:
+            self.context.cascade_report_summary = cascade_report_summary.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_report_summary = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 
 class CascadeReportAtom(CascadeBaseState):
@@ -352,9 +379,18 @@ class CascadeReportAtom(CascadeBaseState):
         if self.context.config_dump_text_blocks:
             super().on_exit()
 
-        # parsed object
-        self.context.cascade_report_atom = cascade_report_atom.parse(self.textbuf.getvalue(),
-                self.context.config_debug_parser)
+        # parse block-separated text
+        try:
+            self.context.cascade_report_atom = cascade_report_atom.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_report_atom = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 
 class CascadeReportDetail(BaseState):
@@ -515,7 +551,19 @@ class CascadeRanger(CascadeBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.cascade_ranger = cascade_ranger.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+
+        # parse block-separated text
+        try:
+            self.context.cascade_ranger = cascade_ranger.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_ranger = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 class CascadeSequin(CascadeBaseState):
     name = 'cascade_sequin'
@@ -525,7 +573,21 @@ class CascadeSequin(CascadeBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.cascade_sequin = cascade_sequin.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+
+
+        # parse block-separated text
+        try:
+            self.context.cascade_sequin = cascade_sequin.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_sequin = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
+
 
 class CascadeOrgiv(CascadeBaseState):
     name = 'cascade_orgiv'
@@ -536,13 +598,26 @@ class CascadeOrgiv(CascadeBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.cascade_orgiv = cascade_orgiv.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+
+        # parse block-separated text
+        try:
+            self.context.cascade_orgiv = cascade_orgiv.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.cascade_orgiv = {}
+                logging.warning(f'Parse error occured in "{self.name} - {self.context.cascade_index_text}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
+
 
 class CascadeEnd(BaseState):
     name = 'cascade_end'
 
     def startswith(self, line):
-        match = re.compile(r'Computation time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+seconds      Cascade storage\s+\d+')
+        match = re.compile(r'Computation time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+(seconds|minutes|hours|days)\s+Cascade storage(\s*\d+|\*{6})')
         return match.fullmatch(line)
 
     def on_exit(self):
@@ -569,7 +644,20 @@ class FinalRangex(RootBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.final_rangex = final_rangex.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+
+
+        # parse block-separated text
+        try:
+            self.context.final_rangex = final_rangex.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.final_rangex = {}
+                logging.warning(f'Parse error occured in "{self.name}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 class FinalSlavex(RootBaseState):
     name = 'final_slavex'
@@ -580,7 +668,20 @@ class FinalSlavex(RootBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.final_slavex = final_slavex.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+
+
+        # parse block-separated text
+        try:
+            self.context.final_slavex = final_slavex.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.final_slavex = {}
+                logging.warning(f'Parse error occured in "{self.name}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 
 class FinalSlavexAtom(RootBaseState):
@@ -592,9 +693,21 @@ class FinalSlavexAtom(RootBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.final_slavex_atom = final_slavex_atom.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
 
 
+        # parse block-separated text
+        try:
+            self.context.final_slavex_atom = final_slavex_atom.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.final_slavex_atom = {}
+                logging.warning(f'Parse error occured in "{self.name}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
+
 class FinalSequex(RootBaseState):
     name = 'final_sequex'
     def startswith(self, line):
@@ -604,9 +717,21 @@ class FinalSequex(RootBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.final_sequex = final_sequex.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
 
 
+        # parse block-separated text
+        try:
+            self.context.final_sequex = final_sequex.parse(self.textbuf.getvalue(),
+                    self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.final_sequex = {}
+                logging.warning(f'Parse error occured in "{self.name}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
+
 class FinalOrgex(RootBaseState):
     name = 'final_orgex'
     def startswith(self, line):
@@ -616,17 +741,29 @@ class FinalOrgex(RootBaseState):
     def on_exit(self):
         if self.context.config_dump_text_blocks:
             super().on_exit()
-        self.context.final_orgex = final_orgex.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
 
+        # parse block-separated text
+        try:
+            self.context.final_orgex = final_orgex.parse(self.textbuf.getvalue(), self.context.config_debug_parser)
+        except Exception as e:
+            if self.context.config_ignore_block_parse_error:
+                self.context.final_orgex = {}
+                logging.warning(f'Parse error occured in "{self.name}", but continued.')
+                logging.warning(str(e), exc_info=True)
+                pass
+            else:
+                raise e
 
 class FinalEnd(RootBaseState):
     name = 'final_end'
 
     def startswith(self, line):
-        match = re.compile(r'Computation time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+seconds      Maximum cascade storage\s+\d+')
+        match = re.compile(r'Computation time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+(seconds|minutes|hours|days)\s+Maximum cascade storage(\s*\d+|\*{6})')
         return match.fullmatch(line)
 
     def on_exit(self):
+        if self.context.config_dump_text_blocks:
+            super().on_exit()
         self.context.on_final_end()
 
 
@@ -634,7 +771,7 @@ class End(BaseState):
     name = 'end'
 
     def startswith(self, line):
-        match = re.compile(r'End of program:  total time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+seconds')
+        match = re.compile(r'End of program:  total time\s+([0-9]*[.])?[0-9]+([eE][-+]?\d+)?\s+(seconds|minutes|hours|days)')
         return match.fullmatch(line)
 
 
@@ -645,6 +782,7 @@ class Context():
                 cascadedir_form='casc{Cascade:05d}-gr{Group:05d}-num{Number:05d}',
                 config_dump_text_blocks=True,
                 config_cascade_table_output='BOTH',
+                config_ignore_block_parse_error=False,
                 config_block_file_form='00.{blockname:s}.txt',
                 config_debug_parser=False):
         """initialize parser
@@ -654,6 +792,7 @@ class Context():
         config_dump_text_blocks: if True, 'as-is' text block is saved, otherwise skipped if it is not needed.
         config_cascade_table_output: chooses whether large output data generated at each cascade
             is written in one file ('BUNDLE'), in each cascade dir('SEPARATE'),  or both('BOTH')
+        config_ignore_block_parse_error: if True, continue even an error occurs during parsing each block.
         config_block_file_form: is fileformat to output parsed block text
         """
 
@@ -664,6 +803,7 @@ class Context():
         self.config_cascade_table_output = config_cascade_table_output
         self.config_debug_parser = config_debug_parser
         self.config_block_file_form = config_block_file_form
+        self.config_ignore_block_parse_error = config_ignore_block_parse_error
 
         # prepare output directory and files
         # output directory
@@ -673,7 +813,8 @@ class Context():
         output_dir.prepare(self.outputdir)
 
         # cascade index
-        self.cascade_index = {'Cascade': 0, 'Group': 0, 'Number': 0}
+        self.cascade_index = {'Cascade':0, 'Group':0, 'Number':0}
+        self.cascade_index_text = 'cascade(0, 0, 0)'
         self.csv_rowindex = [0, 0, 0]
 
         # directry to save each cascade data and summary
@@ -744,9 +885,10 @@ class Context():
     def init_cascade_contexts(self, cascade, group, number):
         '''update cascade_index and cascadedir, then create cascadedir'''
         self.cascade_index = {
-                'Cascade': cascade,
-                'Group': group,
-                'Number': number} 
+                'Cascade':cascade,
+                'Group':group,
+                'Number':number} 
+        self.cascade_index_text = f'cascade({cascade:d}, {group:d}, {number:d})'
         self.csv_rowindex = [cascade, group, number]
         self.cascade_summary = None
         self.cascade_detail_projectile = None
@@ -814,7 +956,7 @@ class Context():
                 csv_cascade_report_detail.writerow(rowdata)
 
             # output to primary recoil ('rec['File'] == 1') data only
-            if not primary_recoil_output and d1[0] == 1:
+            if not primary_recoil_output and d1[0] == self.cascade_detail_projectile['File']['Index']:
                 primary_recoil_output = True
                 if self.csv_cascade_report_detail_primary_all:
                     self.csv_cascade_report_detail_primary_all.writerow(rowdata)
